# implementation of LCS using a more traditional bottom-up dynamic program
# with a two-dimensional table of values.  For simplicity, we will look at
# prefixes of the strings, allowing LCS[j][k] to be the LCS of the first
# j characters of X and the first k characters of Y.

def buildTable(X, Y):
    LCS = [ [0] * (1 + len(Y))  ]       # first row for when j=0
    for j in range(1, 1+len(X)):
        LCS.append( [0] * (1 + len(Y)) )
        for k in range(1, 1+len(Y)):
            if X[j-1] == Y[k-1]:
                LCS[j][k] = 1 + LCS[j-1][k-1]
            else:
                option1 = LCS[j-1][k]
                option2 = LCS[j][k-1]
                LCS[j][k] = max(option1, option2)
    return LCS
    

def reconstructSolution(X, Y, LCS):
    soln = ''
    j = len(X)
    k = len(Y)
    while j > 0 and k > 0:
        if X[j-1] == Y[k-1]:
            soln = X[j-1] + soln
            j = j-1
            k = k-1
        elif LCS[j-1][k] > LCS[j][k-1]:
            j = j-1
        else:
            k = k-1
    return soln



from generate import parseArgs

X,Y = parseArgs()
print(X,Y)

LCS = buildTable(X,Y)
soln = reconstructSolution(X, Y, LCS)

print('LCS "%s" has length %d' % (soln, LCS[len(X)][len(Y)]))

if max(len(X),len(Y)) <= 100:
    print('')
    print('Intermediate table of values:')
    for line in LCS:
        print(line)
